
Copyright (c) 2013, Peter Frank Perroni

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
/*
 * TimeTracker.h
 *
 *  Created on: 28/09/2011
 *  Author: Peter Frank Perroni (pfperroni@inf.ufpr.br)
 */

#ifndef _TIMETRACKER_H_
#define _TIMETRACKER_H_

#include <sstream>
#include <time.h>
#include <sys/time.h>

using namespace std;

/**
 * @brief This class keeps track of the system elapsed time and the CPU clock counter.
 */
class TimeTracker {
	string name;
	clock_t startClock, endClock;
	struct timeval startTime, endTime;
	long double currElapsedTime, currWorkSpent;
	bool processing, paused, done;
	void resetAll();
	void resetCounters();
public:
	TimeTracker(string _name);
	void start();
	void pause();
	void resume();
	long double end();
	long double elapsedTime();
	long double workSpent();
	void reset();
	/**
	 * @brief Return the name of this tracker.
	 * @return The name of this tracker.
	 */
	string getName(){return name;}
	void print(void);
	void print(ostringstream *out);
	void sumWork(TimeTracker *tt);
	void sumTime(TimeTracker *tt);
	void sum(TimeTracker *tt);
};

#endif /* _TIMETRACKER_H_ */
