
Copyright (c) 2013, Torsten Waldmann, Peter Frank Perroni

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
/*
 * PSF.hpp
 *
 *  Created on: 27/05/2012
 *  Author: Peter Frank Perroni (pfperroni@inf.ufpr.br)
 */

#ifndef _PSF_H_
#define _PSF_H_

#include "viennacl/vector.hpp"
#include "viennacl/fft.hpp"
#include "TimeTracker.hpp"
#include "debug.hpp"
#include "Profiling.hpp"

using namespace std;

/**
 * @brief This class represents the PSF.
 *
 * Every instance of this class represents a separate PSF and
 * is responsible for calculating all the PSF information that cannot
 * be calculated on same batch with the others PSFs.
 */
class PSF {
private:
	cl_command_queue command_queue;
	WORD *coefs;
	int n_zernikes, size_fft, img_area, coefs_offset, pupil_offset;
	bool has_coefs, has_pupil;

public:
	cl_mem cl_coefs, cl_pupil, cl_focus, cl_psf, cl_psfe, cl_sum;
	cl_mem cl_fft_psfe;
	cl_mem cl_external_pupil;

	viennacl::vector<WORD> *vn_pupil, *vn_focus, *vn_fft_psfe;

	PSF(cl_context context, cl_command_queue _command_queue, int img_size, int phase_size, int _n_zernikes);

	~PSF();

	void setCoefsPosition(cl_mem _cl_coefs, int offset);

	void setPupilPosition(cl_mem _cl_pupil, int offset);

	void refreshPupil(bool block);

	void pupilToFocusFft();

	void psfeFft();

	WORD* getCoefs();
};

#endif /* _PSF_H_ */
