
Copyright (c) 2013, Torsten Waldmann, Peter Frank Perroni

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
/*
 * psf_estimator.hpp
 *
 *  Created on: 16/07/2012
 *  Author: Peter Frank Perroni (pfperroni@inf.ufpr.br)
 */

#ifndef _PSF_ESTIMATOR_HPP_
#define _PSF_ESTIMATOR_HPP_

#include <ctime>
#include "cl_factory.hpp"
#include "cpso.hpp"
#include "Profiling.hpp"

/**
 * @brief This class is a pool manager for the PSF Estimation methods.
 *
 * Currently, only the search method CPSO (Cooperative Particle Swarm Optimization)
 * is implemented into this class.
 */
class PsfEstimator{
	static CPSO **cpso_pool;
	static bool *cpso_in_use;
	static int cpso_pool_size;
	static omp_lock_t mutex;

	static double *zernikes;
	static int *phase_mask, *diffraction_mask;
	static int phase_size, image_size, n_zernikes, psf_range, n_particles, n_swarms;
	static WORD w, c1, c2, reset_at;

	static CPSO* getCPSO_internal(double *object, double *image, WORD *startp_coefs);

public:
	static void startup(int _cpso_pool_size, double* _zernikes, int *_phase_mask, int *_diffraction_mask,
			int _phase_size, int _image_size, int _n_zernikes, int _psf_range, WORD _w, WORD _c1, WORD _c2,
			WORD _reset_at, int _n_particles, int _n_swarms, vector<int> *_devices);

	static void resetPool(WORD _w, WORD _c1, WORD _c2, WORD _reset_at, int _n_particles, int _n_swarms);

	static void shutdown();

	static CPSO* getCPSO(double *object, double *image);

	static CPSO* getCPSO(double *object, double *image, WORD *startp_coefs);
};

#endif /* _PSF_ESTIMATOR_HPP_ */
